@@ -1,41 +1,45 @@
-// distribution boxbackup-0.10 (svn version: 494)
-//  
-// Copyright (c) 2003 - 2006
-//      Ben Summers and contributors.  All rights reserved.
-//  
-// Redistribution and use in source and binary forms, with or without
-// modification, are permitted provided that the following conditions
-// are met:
-// 1. Redistributions of source code must retain the above copyright
-//    notice, this list of conditions and the following disclaimer.
-// 2. Redistributions in binary form must reproduce the above copyright
-//    notice, this list of conditions and the following disclaimer in the
-//    documentation and/or other materials provided with the distribution.
-// 3. All use of this software and associated advertising materials must 
-//    display the following acknowledgment:
-//        This product includes software developed by Ben Summers.
-// 4. The names of the Authors may not be used to endorse or promote
-//    products derived from this software without specific prior written
-//    permission.
+// distribution boxbackup-0.11.1 (svn version: 2821_2827)
+// Box Backup, http://www.boxbackup.org/
 // 
-// [Where legally impermissible the Authors do not disclaim liability for 
-// direct physical injury or death caused solely by defects in the software 
-// unless it is modified by a third party.]
+// Copyright (c) 2003-2010, Ben Summers and contributors.
+// All rights reserved.
 // 
-// THIS SOFTWARE IS PROVIDED BY THE AUTHORS ``AS IS'' AND ANY EXPRESS OR
-// IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
-// WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
-// DISCLAIMED.  IN NO EVENT SHALL THE AUTHORS BE LIABLE FOR ANY DIRECT,
-// INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
-// (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
-// SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
-// HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
-// STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
-// ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
-// POSSIBILITY OF SUCH DAMAGE.
-//  
-//  
-//  
+// Note that this project uses mixed licensing. Any file with this license
+// attached, or where the code LICENSE-GPL appears on the first line, falls
+// under the "Box Backup GPL" license. See the file COPYING.txt for more
+// information about this license.
+// 
+// ---------------------------------------------------------------------
+// This program is free software; you can redistribute it and/or
+// modify it under the terms of the GNU General Public License
+// as published by the Free Software Foundation; either version 2
+// of the License, or (at your option) any later version.
+// 
+// This program is distributed in the hope that it will be useful,
+// but WITHOUT ANY WARRANTY; without even the implied warranty of
+// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+// GNU General Public License for more details.
+// 
+// You should have received a copy of the GNU General Public License
+// along with this program; if not, write to the Free Software
+// Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
+// 
+// [http://www.gnu.org/licenses/old-licenses/gpl-2.0.html#SEC4]
+// 
+// As a special exception to the GPLv2, the Box Backup Project gives
+// permission to link any code falling under this license (the Box Backup
+// GPL) with any software that can be downloaded from
+// the OpenSSL website [http://www.openssl.org] under either the
+// "OpenSSL License" or the "Original SSLeay License", and to distribute
+// the linked executables under the terms of the "Box Backup GPL" license.
+// 
+// As a special exception to the GPLv2, the Box Backup Project gives
+// permission to link any code falling under this license (the Box Backup
+// GPL) with any version of Microsoft's Volume Shadow Copy Service 7.2 SDK
+// or Microsoft Windows Software Development Kit (SDK), including
+// vssapi.lib, that can be downloaded from the Microsoft website
+// [*.microsoft.com], and to distribute the linked executables under the
+// terms of the "Box Backup GPL" license.
 // --------------------------------------------------------------------------
 //
 // File
@@ -53,16 +57,21 @@
 
 #include <sys/types.h>
 #include <sys/stat.h>
-#include <string.h>
+#include <errno.h>
 #include <limits.h>
+
 #include <algorithm>
+#include <cstring>
 #include <new>
 #include <vector>
+
 #ifdef HAVE_SYS_XATTR_H
 #include <cerrno>
 #include <sys/xattr.h>
 #endif
 
+#include <cstring>
+
 #include "BackupClientFileAttributes.h"
 #include "CommonException.h"
 #include "FileModificationTime.h"
@@ -111,6 +120,9 @@ typedef struct
 typedef struct
 {
 	int32_t uid, gid, mode;
+	#ifdef WIN32
+	int64_t fileCreationTime;
+	#endif
 } attributeHashData;
 
 // Use default packing
@@ -254,12 +266,15 @@ bool BackupClientFileAttributes::operator==(const BackupClientFileAttributes &rA
 //
 // Function
 //		Name:    BackupClientFileAttributes::Compare(const BackupClientFileAttributes &, bool)
-//		Purpose: Compare, optionally ignoring the attribute modification time and/or modification time, and some data which is
-//				 irrelevant in practise (eg file generation number)
+//		Purpose: Compare, optionally ignoring the attribute
+//			 modification time and/or modification time, and some
+//			 data which is irrelevant in practise (eg file
+//			 generation number)
 //		Created: 10/12/03
 //
 // --------------------------------------------------------------------------
-bool BackupClientFileAttributes::Compare(const BackupClientFileAttributes &rAttr, bool IgnoreAttrModTime, bool IgnoreModTime) const
+bool BackupClientFileAttributes::Compare(const BackupClientFileAttributes &rAttr,
+	bool IgnoreAttrModTime, bool IgnoreModTime) const
 {
 	EnsureClearAvailable();
 	rAttr.EnsureClearAvailable();
@@ -267,6 +282,10 @@ bool BackupClientFileAttributes::Compare(const BackupClientFileAttributes &rAttr
 	// Check sizes are the same, as a first check
 	if(mpClearAttributes->GetSize() != rAttr.mpClearAttributes->GetSize())
 	{
+		BOX_TRACE("Attribute Compare: Attributes objects are "
+			"different sizes, cannot compare them: local " <<
+			mpClearAttributes->GetSize() << " bytes, remote " <<
+			rAttr.mpClearAttributes->GetSize() << " bytes");
 		return false;
 	}
 	
@@ -274,28 +293,51 @@ bool BackupClientFileAttributes::Compare(const BackupClientFileAttributes &rAttr
 	// Bytes are checked in network order, but this doesn't matter as we're only checking for equality.
 	attr_StreamFormat *a1 = (attr_StreamFormat*)mpClearAttributes->GetBuffer();
 	attr_StreamFormat *a2 = (attr_StreamFormat*)rAttr.mpClearAttributes->GetBuffer();
-	
-	if(a1->AttributeType != a2->AttributeType
-		|| a1->UID != a2->UID
-		|| a1->GID != a2->GID
-		|| a1->UserDefinedFlags != a2->UserDefinedFlags
-		|| a1->Mode != a2->Mode)
-	{
-		return false;
+
+	#define COMPARE(attribute, message) \
+	if (a1->attribute != a2->attribute) \
+	{ \
+		BOX_TRACE("Attribute Compare: " << message << " differ: " \
+			"local "  << ntoh(a1->attribute) << ", " \
+			"remote " << ntoh(a2->attribute)); \
+		return false; \
 	}
-	
+	COMPARE(AttributeType, "Attribute types");
+	COMPARE(UID, "UIDs");
+	COMPARE(GID, "GIDs");
+	COMPARE(UserDefinedFlags, "User-defined flags");
+	COMPARE(Mode, "Modes");
+
 	if(!IgnoreModTime)
 	{
-		if(a1->ModificationTime != a2->ModificationTime)
+		uint64_t t1 = box_ntoh64(a1->ModificationTime);
+		uint64_t t2 = box_ntoh64(a2->ModificationTime);
+		time_t s1 = BoxTimeToSeconds(t1);
+		time_t s2 = BoxTimeToSeconds(t2);
+		if(s1 != s2)
 		{
+			BOX_TRACE("Attribute Compare: File modification "
+				"times differ: local " <<
+				FormatTime(t1, true) << " (" << s1 << "), "
+				"remote " <<
+				FormatTime(t2, true) << " (" << s2 << ")");
 			return false;
 		}
 	}
-
+	
 	if(!IgnoreAttrModTime)
 	{
-		if(a1->AttrModificationTime != a2->AttrModificationTime)
+		uint64_t t1 = box_ntoh64(a1->AttrModificationTime);
+		uint64_t t2 = box_ntoh64(a2->AttrModificationTime);
+		time_t s1 = BoxTimeToSeconds(t1);
+		time_t s2 = BoxTimeToSeconds(t2);
+		if(s1 != s2)
 		{
+			BOX_TRACE("Attribute Compare: Attribute modification "
+				"times differ: local " <<
+				FormatTime(t1, true) << " (" << s1 << "), "
+				"remote " <<
+				FormatTime(t2, true) << " (" << s2 << ")");
 			return false;
 		}
 	}
@@ -305,8 +347,16 @@ bool BackupClientFileAttributes::Compare(const BackupClientFileAttributes &rAttr
 	if(size > sizeof(attr_StreamFormat))
 	{
 		// Symlink strings don't match. This also compares xattrs
-		if(::memcmp(a1 + 1, a2 + 1, size - sizeof(attr_StreamFormat)) != 0)
+		int datalen = size - sizeof(attr_StreamFormat);
+
+		if(::memcmp(a1 + 1, a2 + 1, datalen) != 0)
 		{
+			std::string s1((char *)(a1 + 1), datalen);
+			std::string s2((char *)(a2 + 1), datalen);
+			BOX_TRACE("Attribute Compare: Symbolic link target "
+				"or extended attributes differ: "
+				"local "  << PrintEscapedBinaryData(s1) << ", "
+				"remote " << PrintEscapedBinaryData(s2));
 			return false;
 		}
 	}
@@ -321,21 +371,30 @@ bool BackupClientFileAttributes::Compare(const BackupClientFileAttributes &rAttr
 // --------------------------------------------------------------------------
 //
 // Function
-//		Name:    BackupClientFileAttributes::ReadAttributes(const char *)
-//		Purpose: Read the attributes of the file, and store them ready for streaming.
-//				 Optionally retrieve the modification time and attribute modification time.
+//		Name:    BackupClientFileAttributes::ReadAttributes(
+//			 const char *Filename, bool ZeroModificationTimes,
+//			 box_time_t *pModTime, box_time_t *pAttrModTime,
+//			 int64_t *pFileSize, InodeRefType *pInodeNumber,
+//			 bool *pHasMultipleLinks)
+//		Purpose: Read the attributes of the file, and store them
+//			 ready for streaming. Optionally retrieve the
+//			 modification time and attribute modification time.
 //		Created: 2003/10/07
 //
 // --------------------------------------------------------------------------
-void BackupClientFileAttributes::ReadAttributes(const char *Filename, bool ZeroModificationTimes, box_time_t *pModTime,
-	box_time_t *pAttrModTime, int64_t *pFileSize, InodeRefType *pInodeNumber, bool *pHasMultipleLinks)
+void BackupClientFileAttributes::ReadAttributes(const char *Filename,
+	bool ZeroModificationTimes, box_time_t *pModTime,
+	box_time_t *pAttrModTime, int64_t *pFileSize,
+	InodeRefType *pInodeNumber, bool *pHasMultipleLinks)
 {
 	StreamableMemBlock *pnewAttr = 0;
 	try
 	{
-		struct stat st;
-		if(::lstat(Filename, &st) != 0)
+		EMU_STRUCT_STAT st;
+		if(EMU_LSTAT(Filename, &st) != 0)
 		{
+			BOX_LOG_SYS_ERROR("Failed to stat file: '" <<
+				Filename << "'");
 			THROW_EXCEPTION(CommonException, OSFileError)
 		}
 		
@@ -382,8 +441,8 @@ void BackupClientFileAttributes::ReadAttributes(const char *Filename, bool ZeroM
 		// to be true (still aborts), but it can at least hold 2^32.
 		if (winTime >= 0x100000000LL || _gmtime64(&winTime) == 0)
 		{
-			::syslog(LOG_ERR, "Invalid Modification Time "
-				"caught for file: %s", Filename);
+			BOX_ERROR("Invalid Modification Time caught for "
+				"file: '" << Filename << "'");
 			pattr->ModificationTime = 0;
 		}
 
@@ -393,8 +452,8 @@ void BackupClientFileAttributes::ReadAttributes(const char *Filename, bool ZeroM
 
 		if (winTime > 0x100000000LL || _gmtime64(&winTime) == 0)
 		{
-			::syslog(LOG_ERR, "Invalid Attribute Modification "
-				"Time caught for file: %s", Filename);
+			BOX_ERROR("Invalid Attribute Modification Time " 
+				"caught for file: '" << Filename << "'");
 			pattr->AttrModificationTime = 0;
 		}
 #endif
@@ -424,7 +483,7 @@ void BackupClientFileAttributes::ReadAttributes(const char *Filename, bool ZeroM
 //		Created: 2003/10/07
 //
 // --------------------------------------------------------------------------
-void BackupClientFileAttributes::FillAttributes(StreamableMemBlock &outputBlock, const char *Filename, struct stat &st, bool ZeroModificationTimes)
+void BackupClientFileAttributes::FillAttributes(StreamableMemBlock &outputBlock, const char *Filename, EMU_STRUCT_STAT &st, bool ZeroModificationTimes)
 {
 	outputBlock.ResizeBlock(sizeof(attr_StreamFormat));
 	attr_StreamFormat *pattr = (attr_StreamFormat*)outputBlock.GetBuffer();
@@ -473,6 +532,7 @@ void BackupClientFileAttributes::FillAttributesLink(StreamableMemBlock &outputBl
 	int linkedToSize = ::readlink(Filename, linkedTo, PATH_MAX);
 	if(linkedToSize == -1)
 	{
+		BOX_LOG_SYS_ERROR("Failed to readlink '" << Filename << "'");
 		THROW_EXCEPTION(CommonException, OSFileError);
 	}
 
@@ -497,7 +557,7 @@ void BackupClientFileAttributes::FillAttributesLink(StreamableMemBlock &outputBl
 void BackupClientFileAttributes::FillExtendedAttr(StreamableMemBlock &outputBlock, const char *Filename)
 {
 #ifdef HAVE_SYS_XATTR_H
-	int listBufferSize = 1000;
+	int listBufferSize = 10000;
 	char* list = new char[listBufferSize];
 
 	try
@@ -563,6 +623,9 @@ void BackupClientFileAttributes::FillExtendedAttr(StreamableMemBlock &outputBloc
 				int valueSize = ::lgetxattr(Filename, attrKey.c_str(), 0, 0);
 				if(valueSize<0)
 				{
+					BOX_LOG_SYS_ERROR("Failed to get "
+						"extended attributes size "
+						"for '" << Filename << "'");
 					THROW_EXCEPTION(CommonException, OSFileError);
 				}
 
@@ -578,6 +641,9 @@ void BackupClientFileAttributes::FillExtendedAttr(StreamableMemBlock &outputBloc
 				valueSize = ::lgetxattr(Filename, attrKey.c_str(), buffer+xattrSize, xattrBufferSize-xattrSize);
 				if(valueSize<0)
 				{
+					BOX_LOG_SYS_ERROR("Failed to get "
+						"extended attributes for " 
+						"'" << Filename << "'");
 					THROW_EXCEPTION(CommonException, OSFileError);
 				}
 				xattrSize += valueSize;
@@ -593,9 +659,25 @@ void BackupClientFileAttributes::FillExtendedAttr(StreamableMemBlock &outputBloc
 
 			outputBlock.ResizeBlock(xattrSize);
 		}
-		else if(listSize<0 && errno!=EOPNOTSUPP && errno!=EACCES)
+		else if(listSize<0)
 		{
-			THROW_EXCEPTION(CommonException, OSFileError);
+			if(errno == EOPNOTSUPP || errno == EACCES)
+			{
+				// fail silently
+			}
+			else if(errno == ERANGE)
+			{
+				BOX_ERROR("Failed to list extended "
+					"attributes of '" << Filename << "': "
+					"buffer too small, not backed up");
+			}
+			else
+			{
+				BOX_LOG_SYS_ERROR("Failed to list extended "
+					"attributes of '" << Filename << "', "
+					"not backed up");
+				THROW_EXCEPTION(CommonException, OSFileError);
+			}
 		}
 	}
 	catch(...)
@@ -607,6 +689,62 @@ void BackupClientFileAttributes::FillExtendedAttr(StreamableMemBlock &outputBloc
 #endif
 }
 
+// --------------------------------------------------------------------------
+//
+// Function
+//		Name:    BackupClientFileAttributes::GetModificationTimes()
+//		Purpose: Returns the modification time embedded in the
+//			 attributes.
+//		Created: 2010/02/24
+//
+// --------------------------------------------------------------------------
+void BackupClientFileAttributes::GetModificationTimes(
+	box_time_t *pModificationTime,
+	box_time_t *pAttrModificationTime) const
+{
+	// Got something loaded
+	if(GetSize() <= 0)
+	{
+		THROW_EXCEPTION(BackupStoreException, AttributesNotLoaded);
+	}
+	
+	// Make sure there are clear attributes to use
+	EnsureClearAvailable();
+	ASSERT(mpClearAttributes != 0);
+
+	// Check if the decrypted attributes are small enough, and the type of attributes stored
+	if(mpClearAttributes->GetSize() < (int)sizeof(int32_t))
+	{
+		THROW_EXCEPTION(BackupStoreException, AttributesNotUnderstood);
+	}
+	int32_t *type = (int32_t*)mpClearAttributes->GetBuffer();
+	ASSERT(type != 0);
+	if(ntohl(*type) != ATTRIBUTETYPE_GENERIC_UNIX)
+	{
+		// Don't know what to do with these
+		THROW_EXCEPTION(BackupStoreException, AttributesNotUnderstood);
+	}
+	
+	// Check there is enough space for an attributes block
+	if(mpClearAttributes->GetSize() < (int)sizeof(attr_StreamFormat))
+	{
+		// Too small
+		THROW_EXCEPTION(BackupStoreException, AttributesNotLoaded);
+	}
+
+	// Get pointer to structure
+	attr_StreamFormat *pattr = (attr_StreamFormat*)mpClearAttributes->GetBuffer();
+
+	if(pModificationTime)
+	{
+		*pModificationTime = box_ntoh64(pattr->ModificationTime);
+	}
+	
+	if(pAttrModificationTime)
+	{
+		*pAttrModificationTime = box_ntoh64(pattr->AttrModificationTime);
+	}
+}
 
 // --------------------------------------------------------------------------
 //
@@ -616,7 +754,8 @@ void BackupClientFileAttributes::FillExtendedAttr(StreamableMemBlock &outputBloc
 //		Created: 2003/10/07
 //
 // --------------------------------------------------------------------------
-void BackupClientFileAttributes::WriteAttributes(const char *Filename) const
+void BackupClientFileAttributes::WriteAttributes(const char *Filename,
+	bool MakeUserWritable) const
 {
 	// Got something loaded
 	if(GetSize() <= 0)
@@ -664,14 +803,15 @@ void BackupClientFileAttributes::WriteAttributes(const char *Filename) const
 		}
 	
 #ifdef WIN32
-		::syslog(LOG_WARNING, 
-			"Cannot create symbolic links on Windows: %s", 
-			Filename);
+		BOX_WARNING("Cannot create symbolic links on Windows: '" <<
+			Filename << "'");
 #else
 		// Make a symlink, first deleting anything in the way
 		::unlink(Filename);
 		if(::symlink((char*)(pattr + 1), Filename) != 0)
 		{
+			BOX_LOG_SYS_ERROR("Failed to symlink '" << Filename <<
+				"' to '" << (char*)(pattr + 1) << "'");
 			THROW_EXCEPTION(CommonException, OSFileError)
 		}
 #endif
@@ -689,12 +829,18 @@ void BackupClientFileAttributes::WriteAttributes(const char *Filename) const
 				// Not a link, use normal chown
 				if(::chown(Filename, ntohl(pattr->UID), ntohl(pattr->GID)) != 0)
 				{
+					BOX_LOG_SYS_ERROR("Failed to change "
+						"owner of file "
+						"'" << Filename << "'");
 					THROW_EXCEPTION(CommonException, OSFileError)
 				}
 			}
 		#else
-			if(::lchown(Filename, ntohl(pattr->UID), ntohl(pattr->GID)) != 0)	// use the version which sets things on symlinks
+			// use the version which sets things on symlinks
+			if(::lchown(Filename, ntohl(pattr->UID), ntohl(pattr->GID)) != 0)
 			{
+				BOX_LOG_SYS_ERROR("Failed to change owner of "
+					"symbolic link '" << Filename << "'");
 				THROW_EXCEPTION(CommonException, OSFileError)
 			}
 		#endif
@@ -717,21 +863,47 @@ void BackupClientFileAttributes::WriteAttributes(const char *Filename) const
 	{
 		// Work out times as timevals
 		struct timeval times[2];
+
+		#ifdef WIN32
+		BoxTimeToTimeval(box_ntoh64(pattr->ModificationTime), 
+			times[1]);
+		BoxTimeToTimeval(box_ntoh64(pattr->AttrModificationTime), 
+			times[0]);
+		// Because stat() returns the creation time in the ctime
+		// field under Windows, and this gets saved in the 
+		// AttrModificationTime field of the serialised attributes,
+		// we subvert the first parameter of emu_utimes() to allow
+		// it to be reset to the right value on the restored file.
+		#else
 		BoxTimeToTimeval(modtime, times[1]);
 		// Copy access time as well, why not, got to set it to something
 		times[0] = times[1];
-		// Attr modification time will be changed anyway, nothing that can be done about it
+		// Attr modification time will be changed anyway, 
+		// nothing that can be done about it
+		#endif
 		
 		// Try to apply
 		if(::utimes(Filename, times) != 0)
 		{
-			THROW_EXCEPTION(CommonException, OSFileError)
+			BOX_LOG_SYS_WARNING("Failed to change times of "
+				"file '" << Filename << "' to ctime=" <<
+				BOX_FORMAT_TIMESPEC(times[0]) << ", mtime=" << 
+				BOX_FORMAT_TIMESPEC(times[1]));
 		}
 	}
-	
-	// Apply everything else... (allowable mode flags only)
-	if(::chmod(Filename, mode & (S_IRWXU | S_IRWXG | S_IRWXO | S_ISUID | S_ISGID | S_ISVTX)) != 0)	// mode must be done last (think setuid)
+
+	if (MakeUserWritable)
 	{
+		mode |= S_IRWXU;
+	}
+
+	// Apply everything else... (allowable mode flags only)
+	// Mode must be done last (think setuid)
+	if(::chmod(Filename, mode & (S_IRWXU | S_IRWXG | S_IRWXO | S_ISUID
+		| S_ISGID | S_ISVTX)) != 0)
+	{
+		BOX_LOG_SYS_ERROR("Failed to change permissions of file "
+			"'" << Filename << "'");
 		THROW_EXCEPTION(CommonException, OSFileError)
 	}
 }
@@ -846,6 +1018,8 @@ void BackupClientFileAttributes::WriteExtendedAttr(const char *Filename, int xat
 		// FIXME: Warn on EOPNOTSUPP
 		if(::lsetxattr(Filename, key, buffer+xattrOffset, valueSize, 0)!=0 && errno!=EOPNOTSUPP)
 		{
+			BOX_LOG_SYS_ERROR("Failed to set extended attributes "
+				"on file '" << Filename << "'");
 			THROW_EXCEPTION(CommonException, OSFileError);
 		}
 
@@ -1001,14 +1175,18 @@ void BackupClientFileAttributes::SetAttributeHashSecret(const void *pSecret, int
 // --------------------------------------------------------------------------
 //
 // Function
-//		Name:    BackupClientFileAttributes::GenerateAttributeHash(struct stat &, const std::string &, const std::string &)
-//		Purpose: Generate a 64 bit hash from the attributes, used to detect changes.
-//				 Include filename in the hash, so that it changes from one file to another,
-//				 so don't reveal identical attributes.
+//		Name:    BackupClientFileAttributes::GenerateAttributeHash(
+//			 struct stat &, const std::string &,
+//			 const std::string &)
+//		Purpose: Generate a 64 bit hash from the attributes, used to
+//			 detect changes. Include filename in the hash, so
+//			 that it changes from one file to another, so don't
+//			 reveal identical attributes.
 //		Created: 25/4/04
 //
 // --------------------------------------------------------------------------
-uint64_t BackupClientFileAttributes::GenerateAttributeHash(struct stat &st, const std::string &filename, const std::string &leafname)
+uint64_t BackupClientFileAttributes::GenerateAttributeHash(EMU_STRUCT_STAT &st,
+	const std::string &filename, const std::string &leafname)
 {
 	if(sAttributeHashSecretLength == 0)
 	{
@@ -1023,6 +1201,16 @@ uint64_t BackupClientFileAttributes::GenerateAttributeHash(struct stat &st, cons
 	hashData.gid = htonl(st.st_gid);
 	hashData.mode = htonl(st.st_mode);
 
+	#ifdef WIN32
+	// On Windows, the "file attribute modification time" is the
+	// file creation time, and we want to back this up, restore
+	// it and compare it.
+	//
+	// On other platforms, it's not very important and can't
+	// reliably be set to anything other than the current time.
+	hashData.fileCreationTime = box_hton64(st.st_ctime);
+	#endif
+
 	StreamableMemBlock xattr;
 	FillExtendedAttr(xattr, filename.c_str());
 
@@ -1031,7 +1219,7 @@ uint64_t BackupClientFileAttributes::GenerateAttributeHash(struct stat &st, cons
 	digest.Add(&hashData, sizeof(hashData));
 	digest.Add(xattr.GetBuffer(), xattr.GetSize());
 	digest.Add(leafname.c_str(), leafname.size());
-	digest.Add(sAttributeHashSecret, sAttributeHashSecretLength);
+	digest.Add(sAttributeHashSecret, sAttributeHashSecretLength);	
 	digest.Finish();
 	
 	// Return the first 64 bits of the hash
